------------------------------------------------------------------------------------
-- Company: Univ Lyon, UJM-Saint-Etienne, CNRS, Laboratoire Hubert Curien UMR 5516, F-42023, SAINT-ETIENNE, France
-- Creator: Cédric Marchand  
-- 
-- Create Date:    18:07:09 11/11/2015 
-- Design Name: 
-- Module Name:    Datapath - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

LIBRARY work;
USE work.API_pkg.ALL;

entity Datapath is
  PORT(
    clk_i : in std_logic;
    rstb_i : in std_logic;
    --Input
    data_i    : in std_logic_vector(7 downto 0);
    key_i     : in std_logic_vector(7 downto 0);
    --Control
    mode_i    : in std_logic;
    key_sel   : in std_logic;
    data_sel  : in std_logic_vector(1 downto 0);
    ena_t     : in std_logic;
    --Output
    data_o    : out std_logic_vector(7 downto 0);
    T_o       : out std_logic_vector(7 downto 0)
  );
end Datapath;

architecture Behavioral of Datapath is

  COMPONENT function_fa IS
  PORT(
    mode_i   : IN STD_LOGIC;                     --mode
    L1       : IN STD_LOGIC_VECTOR(24 DOWNTO 0); --register L1
    L2_i     : IN STD_LOGIC_VECTOR(2 DOWNTO 0);  --LSB of L2
    IR       : IN STD_LOGIC;                     --irregular rule
    k        : IN STD_LOGIC;                     --subkey bit ka
    fa_out   : OUT STD_LOGIC
    );
  END COMPONENT;
  
  COMPONENT function_fb IS
  PORT(
    mode_i : IN STD_LOGIC;                     --mode
    L2     : IN STD_LOGIC_VECTOR(38 DOWNTO 0); --register L2
    L1_i   : IN STD_LOGIC_VECTOR(2 DOWNTO 0);                     --LSB of L1
    k      : IN STD_LOGIC;                     --subkey bit kb
    fb_out : OUT STD_LOGIC
    );
  END COMPONENT;
  
  COMPONENT round_count IS
  PORT(
    mode_i : IN STD_LOGIC;                       --mode
    T_in   : IN STD_LOGIC_VECTOR(7 DOWNTO 0);    --round counter in
    T_out  : OUT STD_LOGIC_VECTOR(7 DOWNTO 0)    --round counter out
    );
  END COMPONENT;
  
  COMPONENT keyschedule
	PORT(
		K : IN std_logic_vector(79 downto 0);
		T : IN std_logic_vector(7 downto 0);          
		ka : OUT std_logic;
		kb : OUT std_logic
		);
	END COMPONENT;
  
  --signals
  --regsiters
  signal key_reg    : std_logic_vector(key_size-1 downto 0);
  signal state_reg  : std_logic_vector(state_size-1 downto 0);
  
  signal T_reg          : std_logic_vector(7 downto 0);
  signal T_s            : std_logic_vector(7 downto 0);
  signal fa_s           : std_logic;
  signal fb_s           : std_logic;
  signal ka_s           : std_logic;
  signal kb_s           : std_logic;
  
begin
  --key process
  K_proc : process(clk_i, key_sel)
  begin
    if rising_edge(clk_i) then
      if key_sel = '1' then
        key_reg <= key_i & key_reg(key_size-1 downto 8);
      else
        key_reg <= key_reg;
      end if;
    end if;
  end process;
  
  --data process
  D_proc : process(clk_i, data_sel)
  begin
    if rising_edge(clk_i) then
      if data_sel = "01" then
        state_reg <= data_i & state_reg(state_size-1 downto 8);
      elsif data_sel = "10" then
        state_reg <=  state_reg(62 downto 39) & fb_s & state_reg(37 downto 0) & fa_s;
      elsif data_sel = "11" then
        state_reg <= fa_s & state_reg(63 downto 40) & fb_s & state_reg(38 downto 1);
      else
        state_reg <= state_reg;
      end if;
    end if;
  end process;
  
  T_proc : process(rstb_i, clk_i, ena_t)
  begin
    if rstb_i = '0' then
      T_reg <= (others => '1');
    elsif rising_edge(clk_i) then
      if ena_t = '1' then
        T_reg <= T_s;
      else
        T_reg <= T_reg;
      end if;
    end if;
  end process;
  
  data_o <= state_reg(7 downto 0);

  -- cipher component
  Inst_function_fa: function_fa PORT MAP(
		mode_i => mode_i,
		L1 => state_reg(63 downto 39),
		L2_i => state_reg(2 downto 0),
		IR => T_s(7),
		k => ka_s,
		fa_out => fa_s
	);

  Inst_function_fb: function_fb PORT MAP(
		mode_i => mode_i,
		L2 => state_reg(38 downto 0),
		L1_i => state_reg(41 downto 39),
		k => kb_s,
		fb_out => fb_s
	);
  
  i_round_count : round_count PORT MAP (
    mode_i => mode_i,
    T_in   => T_reg,
    T_out  => T_s
    );
    
  -- keyschedule
  Inst_keyschedule: keyschedule PORT MAP(
		K => Key_reg,
		T => T_reg,
		ka => ka_s,
		kb => kb_s
	);
  
end Behavioral;

