------------------------------------------------------------------------------------
-- Company: Univ Lyon, UJM-Saint-Etienne, CNRS, Laboratoire Hubert Curien UMR 5516, F-42023, SAINT-ETIENNE, France
-- Creator: Cédric Marchand  
-- 
-- Create Date:    10:50:44 10/29/2015 
-- Design Name: 
-- Module Name:    Top_level - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

LIBRARY work;
USE work.API_pkg.ALL;

entity Top_level is
  port(
    rstb_i    : in std_logic;
    clk_i     : in std_logic;
    --input
    control_i : in std_logic_vector(1 downto 0);
    data_i    : in std_logic_vector(7 downto 0);
    key_i     : in std_logic_vector(7 downto 0);
    --output
    busy_o    : out std_logic;
    key_valid : out std_logic;
    done_o    : out std_logic;
    data_o    : out std_logic_vector(7 downto 0)
  );
end Top_level;

architecture Behavioral of Top_level is

  --Control unit
  COMPONENT control_unit
	generic (
    state_size  : in integer := 64;
    key_size    : in integer := 80
  );
  port(
    clk_i     : in std_logic;
    rstb_i    : in std_logic;
    --instruction
    control_i : in std_logic_vector(1 downto 0);
    --output flag
    key_ready : out std_logic;
    busy_o    : out std_logic;
    done_o    : out std_logic;
    --control signal
    T         : in std_logic_vector(7 downto 0);
    ena_t     : out std_logic;
    mode_o    : out std_logic;
    key_sel   : out std_logic;
    data_sel  : out std_logic_vector(1 downto 0)
    );
	END COMPONENT;
  
  COMPONENT Datapath
	PORT(
		clk_i : IN std_logic;
		rstb_i : IN std_logic;
		data_i : IN std_logic_vector(7 downto 0);
		key_i : IN std_logic_vector(7 downto 0);
		mode_i : IN std_logic;
		key_sel : IN std_logic;
		data_sel : IN std_logic_vector(1 downto 0);
    ena_t    : IN std_logic;
		data_o : OUT std_logic_vector(7 downto 0);
    T_o : OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;
  
  --internal control signals
  signal key_sel_s  : std_logic;
  signal data_sel_s : std_logic_vector(1 downto 0);
  signal done_s     : std_logic;
  signal mode_s     : std_logic;
  signal ena_t_s    : std_logic;
  
  --process signals
  signal T_s            : std_logic_vector(7 downto 0);
     
begin

  Inst_control_unit: control_unit PORT MAP(
		clk_i     => clk_i,
    rstb_i    => rstb_i,
		control_i => control_i,
		key_ready => key_valid,
		busy_o    => busy_o,
		done_o    => done_s,
		mode_o    => mode_s,
    T         => T_s,
    ena_t     => ena_t_s,
    key_sel   => key_sel_s,
    data_sel  => data_sel_s
	);
  
  Inst_Datapath: Datapath PORT MAP(
		clk_i => clk_i,
    rstb_i    => rstb_i,
		data_i => data_i,
		key_i => key_i,
		mode_i => mode_s,
		key_sel => key_sel_s,
		data_sel => data_sel_s,
    ena_t   => ena_t_s,
    T_o => T_s,
		data_o => data_o
	);
  
end Behavioral;

